//! Example of using the Git module with Rhai
//!
//! This example demonstrates how to use the Git module functions
//! through the Rhai scripting language.

use sal::rhai::{self, Engine};

fn main() -> Result<(), Box<dyn std::error::Error>> {
    // Create a new Rhai engine
    let mut engine = Engine::new();
    
    // Register SAL functions with the engine
    rhai::register(&mut engine)?;
    
    // Run a Rhai script that uses Git functions
    let script = r#"
        // Print a header
        print("=== Testing Git Module Functions ===\n");

        // Create a new GitTree object
        let home_dir = env("HOME");
        let git_tree = gittree_new(`${home_dir}/code`);
        print(`Created GitTree with base path: ${home_dir}/code`);

        // Test list method
        print("\nListing git repositories...");
        let repos = git_tree.list();
        print(`Found ${repos.len()} repositories`);

        // Print the first few repositories
        if repos.len() > 0 {
            print("First few repositories:");
            let count = if repos.len() > 3 { 3 } else { repos.len() };
            for i in range(0, count) {
                print(`  - ${repos[i]}`);
            }
        }

        // Test find method
        if repos.len() > 0 {
            print("\nTesting repository search...");
            // Extract a part of the first repo name to search for
            let repo_path = repos[0];
            let parts = repo_path.split("/");
            let repo_name = parts[parts.len() - 1];
            
            print(`Searching for repositories containing "${repo_name}"`);
            let matching = git_tree.find(repo_name + "*");
            
            print(`Found ${matching.len()} matching repositories`);
            for repo in matching {
                print(`  - ${repo}`);
            }
            
            // Test get method
            print("\nTesting get method...");
            let git_repos = git_tree.get(repo_name);
            print(`Found ${git_repos.len()} GitRepo objects`);
            
            // Test GitRepo methods
            if git_repos.len() > 0 {
                let git_repo = git_repos[0];
                print(`\nTesting GitRepo methods on: ${git_repo.path()}`);
                
                // Check if a repository has changes
                print("Checking for changes in repository...");
                let has_changes = git_repo.has_changes();
                print(`Repository has changes: ${has_changes}`);
                
                // Test method chaining (only if there are no changes to avoid errors)
                if !has_changes {
                    print("\nTesting method chaining (pull)...");
                    let result = git_repo.pull();
                    print("Pull operation completed successfully");
                }
            }
        }

        print("\n=== Git Module Test Complete ===");
    "#;
    
    // Evaluate the script
    match engine.eval::<()>(script) {
        Ok(_) => println!("Script executed successfully"),
        Err(e) => eprintln!("Script execution error: {}", e),
    }
    
    Ok(())
}